@@ -1,9 +1,9 @@
 use ansi_term::Style;
 use crossterm::{cursor::MoveLeft, ExecutableCommand};
 use dirs::home_dir;
+use serde::{Deserialize, Serialize};
 use text_io::read;
 use tokio::{join, spawn};
-use toml::Value;
 
 use egg_mode::{
     auth::{access_token, authorize_url, request_token},
@@ -13,30 +13,47 @@ use egg_mode::{
 };
 
 use std::{
+    clone::Clone,
     fs,
     future::Future,
     io::{stdout, Write},
     process::exit,
 };
 
+#[derive(Clone, Deserialize, Serialize)]
+struct Keys {
+    key: String,
+    secret: String,
+    access_token: Option<String>,
+    access_token_secret: Option<String>,
+}
+
 type TwitterResult = Result<(), TwitterErrors>;
 
 async fn authenticate() -> Token {
+    let auth_msg = Style::new().italic().paint("Authenticating...");
+    println!("{}", auth_msg);
     let key_file = home_dir().unwrap().join(".config/qt/keys.toml");
-    let key_map = fs::read_to_string(key_file)
-        .unwrap()
-        .parse::<Value>()
-        .unwrap();
-    let (key, secret) = (
-        key_map["key"].as_str().unwrap().to_string(),
-        key_map["secret"].as_str().unwrap().to_string(),
-    );
-    let consumer_pair = KeyPair::new(key, secret);
+    let key_map: Keys = toml::from_str(&fs::read_to_string(&key_file).unwrap()).unwrap();
+    if key_map.access_token.is_some() {
+        let consumer_pair = KeyPair::new(key_map.key, key_map.secret);
+        let access_token_pair = KeyPair::new(
+            key_map.access_token.unwrap(),
+            key_map.access_token_secret.unwrap(),
+        );
+
+        return Token::Access {
+            consumer: consumer_pair,
+            access: access_token_pair,
+        };
+    }
+    let mut config_file = key_map.clone();
+    let consumer_pair = KeyPair::new(key_map.key, key_map.secret);
 
     let auth_token = request_token(&consumer_pair, "oob").await.unwrap();
     let auth_url = authorize_url(&auth_token);
     print!(
-        "Go to this URL to authenticate: {}\n\nEnter PIN: ",
+        "\nGo to this URL to authenticate: {}\n\nEnter PIN: ",
         auth_url
     );
     stdout().flush().unwrap();
@@ -46,6 +63,19 @@ async fn authenticate() -> Token {
         .await
         .unwrap();
 
+    match token {
+        Token::Access {
+            consumer: _,
+            access: ref a,
+        } => {
+            config_file.access_token = Some(a.key.to_string());
+            config_file.access_token_secret = Some(a.secret.to_string());
+        }
+        Token::Bearer(_) => panic!(),
+    };
+    let key_toml = toml::to_string(&config_file).unwrap();
+    fs::write(key_file, key_toml).unwrap();
+
     token
 }
 
